/*****************************************************************************/
/*                                                                           */
/* TIFFSDK.C                                                                 */
/*                                                                           */
/* Version: 1.1                                                              */
/*                                                                           */
/* This file is the 'magic' behind the TIFF SDK.  So long as this file is    */
/* included in your project's build, your application's API calls will be    */
/* automatically linked at runtime, and your main() function called.         */
/*                                                                           */
/* Note that this file currently contains a selection of commonly-used API   */
/* calls.  If you want to use an API function that isn't already in this 		 */
/* file, then you will need to add those functions to this file, as follows: */
/*                                                                           */
/*  * Check that the API call you want to use is supported by the SDK.       */
/*    (The list of supported calls is given in the file 'functions.txt').    */
/*                                                                           */
/*  * Add a call to resolveSyscall for the function you want to use, in      */
/*    the init() function below, following the pattern shown.  The second    */
/*    parameter is the NID code for the function, obtained from 						 */
/* 		functions.txt. 																												 */
/*                                                                           */
/*  * Add a stub function to the set just before the init() function,        */
/*    following the pattern shown.                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* DO NOT CHANGE THIS SECTION                                                */
/*                                                                           */
/*****************************************************************************/
#include <pspuser.h>
#include <pspctrl.h>
#include <malloc.h>
#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <pspgu.h>
#include <pspgum.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <png.h>
#include <pspiofilemgr.h>

typedef unsigned long (*PFN_RESOLVEFUNC)(unsigned long);
void (*tiffsdkKernelDcacheWritebackAll)(void);
#define FUNC_TO_JMP(A)  (((((unsigned long)((void*)(A))) & 0x0FFFFFFF) >> 2) | 0x08000000L)
PFN_RESOLVEFUNC findFunctionByNID;

unsigned long failedNID;

/*****************************************************************************/
/* Set by the linker to be the first instruction                             */
/*****************************************************************************/
void _start(unsigned long, unsigned long *) __attribute__ ((section (".text.start")));
void _start(unsigned long arglen, unsigned long *argp)
{
	findFunctionByNID = (PFN_RESOLVEFUNC*)argp[1];
	init();
	main();
}

/*****************************************************************************/
/* Flash screen 10 times                                                     */
/*****************************************************************************/
void flashScreen(int cola, int colb)
{
  {
    int i;

    for (i=0; i<10; i++)
    {
      {
        long *lptr = 0x44000000;

        while (lptr < 0x44220000)
        {
          *lptr++ = cola;
        }

        lptr = 0x44000000;

        while (lptr < 0x44220000)
        {
          *lptr++ = colb;
        }
      }
    }
  }
}


/*****************************************************************************/
/* Retrieves the address to jump to for a given NID, then writes the         */
/* necessary opcodes into the stub function.                                 */
/*****************************************************************************/
void linkFunctionByNID(unsigned long nid, void *funcaddr)
{
	unsigned long realFunc = findFunctionByNID(nid);

	if (!realFunc)
	{
	/*************************************************************************/
	/* We got an unresolved function.  We can't guarantee that we currently  */
	/* have enough functions resolved to display anything meaningful, so we  */
	/* just flash the screen as a warning.                                   */
	/*************************************************************************/
		flashScreen(0, 0x000000FFL);
		failedNID = nid;
		
		// Safety patch the stub to just return -1 to avoid crash
		*((unsigned long*)funcaddr) = 0x03e00008;
		*(((unsigned long*)funcaddr) + 1) = 0x2402ffff;
		return;
	}

	if(nid==0x79d1c3faL) tiffsdkKernelDcacheWritebackAll = (void*)realFunc;
	
	*((unsigned long*)funcaddr) = FUNC_TO_JMP(realFunc);
	*(((unsigned long*)funcaddr) + 1) = 0;
}

/*****************************************************************************/
/*                                                                           */
/* END OF 'DO NOT CHANGE THIS SECTION'                                       */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* Function stubs start here.  All SCE API functions that are used must      */
/* have a corresponding function stub.                                       */
/*****************************************************************************/
int sceCtrlGetSamplingMode(int *pmode)
{ asm ("syscall 0x0000;"); };

int sceCtrlPeekBufferPositive(SceCtrlData *pad_data, int count)
{ asm ("syscall 0x0000;"); };

int sceCtrlReadBufferPositive(SceCtrlData *pad_data, int count)
{ asm ("syscall 0x0000;"); };

int sceCtrlSetSamplingMode(int mode)
{ asm ("syscall 0x0000;"); };

void sceDisplaySetFrameBuf(void *topaddr, int bufferwidth, int pixelformat, int sync)
{ asm ("syscall 0x0000;"); };

int sceDisplaySetMode(int mode, int width, int height)
{ asm ("syscall 0x0000;"); };

int sceDisplayWaitVblankStart(void)
{ asm ("syscall 0x0000;"); };

int sceIoClose(SceUID fd)
{ asm ("syscall 0x0000;"); };

int sceIoDclose(SceUID fd)
{ asm ("syscall 0x0000;"); };

SceUID sceIoDopen(const char *dirname)
{ asm ("syscall 0x0000;"); };

int sceIoDread(SceUID fd, SceIoDirent *dir)
{ asm ("syscall 0x0000;"); };

SceOff sceIoLseek(SceUID fd, SceOff offset, int whence)
{ asm ("syscall 0x0000;"); };

int sceIoMkdir(const char *dir, SceMode mode)
{ asm ("syscall 0x0000;"); };

SceUID sceIoOpen(const char *file, int flags, SceMode mode)
{ asm ("syscall 0x0000;"); };

int sceIoRead(SceUID fd, void *data, SceSize size)
{ asm ("syscall 0x0000;"); };

int sceIoRemove(const char *file)
{ asm ("syscall 0x0000;"); };

int sceIoRename(const char *oldname, const char *newname)
{ asm ("syscall 0x0000;"); };

int sceIoRmdir(const char *path)
{ asm ("syscall 0x0000;"); };

int sceIoWrite(SceUID fd, const void *data, SceSize size)
{ asm ("syscall 0x0000;"); };

int sceKernelCancelEventFlag(SceUID evid)
{ asm ("syscall 0x0000;"); };

int sceKernelCancelSema(SceUID semaid)
{ asm ("syscall 0x0000;"); };

int sceKernelClearEventFlag(SceUID evid, u32 bits)
{ asm ("syscall 0x0000;"); };

int sceKernelCreateCallback(const char *name, SceKernelCallbackFunction func,
                            void *arg)
{ asm ("syscall 0x0000;"); };
SceUID sceKernelCreateEventFlag(const char *name, int attr, int bits,
                                SceKernelEventFlagOptParam *opt)
{ asm ("syscall 0x0000;"); };

SceUID sceKernelCreateSema(const char *name, SceUInt attr, int initVal,
                           int maxVal, SceKernelSemaOptParam *option)
{ asm ("syscall 0x0000;"); };
SceUID sceKernelCreateThread(const char *name, SceKernelThreadEntry entry,
                             int initPriority, int stackSize, SceUInt attr,
                             SceKernelThreadOptParam *option)
{ asm ("syscall 0x0000;"); };

void sceKernelDcacheWritebackAll(void)
{ asm ("syscall 0x0000;"); };

void sceKernelDcacheWritebackInvalidateRange(const void *p, unsigned int size)
{ asm ("syscall 0x0000;"); };

void sceKernelDcacheWritebackRange(const void *p, unsigned int size)
{ asm ("syscall 0x0000;"); };

void sceKernelDelayThread(SceUInt delay)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteCallback(int cbid)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteEventFlag(int evid)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteFpl(SceUID uid)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteSema(SceUID semaid)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteThread(SceUID thid)
{ asm ("syscall 0x0000;"); };

int sceKernelDeleteVpl(SceUID uid)
{ asm ("syscall 0x0000;"); };

int sceKernelExitDeleteThread(int status)
{ asm ("syscall 0x0000;"); };

int sceKernelExitThread(int status)
{ asm ("syscall 0x0000;"); };

int sceKernelFreeFpl(SceUID uid, void *data)
{ asm ("syscall 0x0000;"); };

int sceKernelFreeVpl(SceUID uid, void *data)
{ asm ("syscall 0x0000;"); };

int sceKernelGetThreadId(void)
{ asm ("syscall 0x0000;"); };

int sceKernelPollEventFlag(int evid, u32 bits, u32 wait, u32 *outBits)
{ asm ("syscall 0x0000;"); };

int sceKernelPollSema(SceUID semaid, int signal)
{ asm ("syscall 0x0000;"); };

void sceKernelPowerTick()
{ asm ("syscall 0x0000;"); };

int sceKernelSetEventFlag(SceUID evid, u32 bits)
{ asm ("syscall 0x0000;"); };

int sceKernelSignalSema(SceUID semaid, int signal)
{ asm ("syscall 0x0000;"); };

int sceKernelStartThread(SceUID thid, SceSize arglen, void *argp)
{ asm ("syscall 0x0000;"); };

int sceKernelTerminateDeleteThread(SceUID thid)
{ asm ("syscall 0x0000;"); };

int sceKernelWaitEventFlag(int evid, u32 bits, u32 wait, u32 *outBits, SceUInt *timeout)
{ asm ("syscall 0x0000;"); };

int sceKernelWaitSema(SceUID semaid, int signal, SceUInt *timeout)
{ asm ("syscall 0x0000;"); };

int sceKernelWaitThreadEnd(SceUID thid, SceUInt *timeout)
{ asm ("syscall 0x0000;"); };

int sceRtcGetCurrentTick(u64 *tick)
{ asm ("syscall 0x0000;"); };

int vshCtrlReadBufferPositive(SceCtrlData *pad_data, int count)
{ asm ("syscall 0x0000;"); };

void vshKernelExitVSHVSH(void *unk_struct)
{ asm ("syscall 0x0000;"); };

int rand()
{ asm ("syscall 0x0000;"); }

float sqrtf(float x)
{ asm ("syscall 0x0000;"); }

float sinf(float x)
{ asm ("syscall 0x0000;"); }

float cosf(float x)
{ asm ("syscall 0x0000;"); }

int sceKernelDevkitVersion()
{ asm ("syscall 0x0000;"); };

int sceAudioOutputPannedBlocking(int channel, int leftvol, int rightvol, void *buffer)
{ asm ("syscall 0x0000;"); };

int sceAudioOutputPanned(int channel, int leftvol, int rightvol, void *buffer)
{ asm ("syscall 0x0000;"); };

int sceAudioChReserve(int channel, int samplecount, int format)
{ asm ("syscall 0x0000;"); };

int sceAudioChRelease(int channel)
{ asm ("syscall 0x0000;"); };

// GU FUNCTIONS

int memcmp(const void *s1, const void *s2, size_t n)
{ asm ("syscall 0x0000;"); }

void *memcpy(void *s1, const void *s2, size_t n)
{ asm ("syscall 0x0000;"); }

void free(void *ptr)
{ asm ("syscall 0x0000;"); }

void *malloc(unsigned int size)
{ asm ("syscall 0x0000;"); }

int sceCtrlSetSamplingCycle( int cycle )
{ asm ("syscall 0x0000;"); };

/*void sceGuInit (void)
{ asm ("syscall 0x0000;"); }

void sceGuStart (int cid, void *list)
{ asm ("syscall 0x0000;"); }

void sceGuDrawBuffer (int psm, void *fbp, int fbw)
{ asm ("syscall 0x0000;"); }

void sceGuDispBuffer (int width, int height, void *dispbp, int dispbw)
{ asm ("syscall 0x0000;"); }

void sceGuDepthBuffer (void *zbp, int zbw)
{ asm ("syscall 0x0000;"); }

void sceGuOffset (unsigned int x, unsigned int y)
{ asm ("syscall 0x0000;"); }

void sceGuViewport (int cx, int cy, int width, int height)
{ asm ("syscall 0x0000;"); }

void sceGuDepthRange (int near, int far)
{ asm ("syscall 0x0000;"); }

void sceGuScissor (int x, int y, int w, int h)
{ asm ("syscall 0x0000;"); }

void sceGuEnable (int state)
{ asm ("syscall 0x0000;"); }

void sceGuDisable (int state)
{ asm ("syscall 0x0000;"); }

void sceGuDepthFunc (int function)
{ asm ("syscall 0x0000;"); }

void sceGuFrontFace (int order)
{ asm ("syscall 0x0000;"); }

void sceGuShadeModel (int mode)
{ asm ("syscall 0x0000;"); }

void sceGuTexMode(int tpsm, int maxmips, int a2, int swizzle)
{ asm ("syscall 0x0000;"); }

void sceGuTexFunc (int tfx, int tcc)
{ asm ("syscall 0x0000;"); }

void sceGuTexFilter (int min, int mag)
{ asm ("syscall 0x0000;"); }

void sceGuTexScale (float u, float v)
{ asm ("syscall 0x0000;"); }

void sceGuTexOffset (float u, float v)
{ asm ("syscall 0x0000;"); }

int sceGuFinish (void)
{ asm ("syscall 0x0000;"); }

int sceGuSync (int mode, int a1)
{ asm ("syscall 0x0000;"); }

int sceGuDisplay (int state)
{ asm ("syscall 0x0000;"); }

void sceGumMatrixMode(int mode)
{ asm ("syscall 0x0000;"); }

void sceGumLoadIdentity(void)
{ asm ("syscall 0x0000;"); }; 

void sceGumPerspective(float fovy, float aspect, float near, float far)
{ asm ("syscall 0x0000;"); }; 

void sceGuClearColor (unsigned int color)
{ asm ("syscall 0x0000;"); }

void sceGuClearDepth (unsigned int depth)
{ asm ("syscall 0x0000;"); }

void sceGuClear(int flags)
{ asm ("syscall 0x0000;"); }
void sceGumTranslate(const ScePspFVector3* v)
{ asm ("syscall 0x0000;"); }; 
void sceGuTexImage (int mipmap, int width, int height, int tbw, const void *tbp)
{ asm ("syscall 0x0000;"); }
void sceGumRotateXYZ(const ScePspFVector3* v)
{ asm ("syscall 0x0000;"); }
void sceGumDrawArray (int prim, int vtype, int count, const void* indices, const void* vertices)
{ asm ("syscall 0x0000;"); }
void *sceGuSwapBuffers (void)
{ asm ("syscall 0x0000;"); }
void sceGumRotateX (float angle)
{ asm ("syscall 0x0000;"); }*/

// END GU FUNCTIONS
/*****************************************************************************/
/* Patches everything up ready for main() to execute.                        */
/*****************************************************************************/
init()
{
        failedNID = 0;

        /***************************************************************************/
        /* All imported functions must appear in this list.   This patches up the  */
        /* function calls to the correct addresses.                                */
        /***************************************************************************/
        linkFunctionByNID(0xda6b76a1L, &sceCtrlGetSamplingMode);
        linkFunctionByNID(0x3a622550L, &sceCtrlPeekBufferPositive);
        linkFunctionByNID(0x1f803938L, &sceCtrlReadBufferPositive);
        linkFunctionByNID(0x1f4011e6L, &sceCtrlSetSamplingMode);
        linkFunctionByNID(0x289d82feL, &sceDisplaySetFrameBuf);
        linkFunctionByNID(0x0e20f177L, &sceDisplaySetMode);
        linkFunctionByNID(0x984c27e7L, &sceDisplayWaitVblankStart);
        linkFunctionByNID(0x810c4bc3L, &sceIoClose);
        linkFunctionByNID(0xeb092469L, &sceIoDclose);
        linkFunctionByNID(0xb29ddf9cL, &sceIoDopen);
        linkFunctionByNID(0xe3eb004cL, &sceIoDread);
        linkFunctionByNID(0x27eb27b8L, &sceIoLseek);
        linkFunctionByNID(0x06a70004L, &sceIoMkdir);
        linkFunctionByNID(0x109f50bcL, &sceIoOpen);
        linkFunctionByNID(0x6a638d83L, &sceIoRead);
        linkFunctionByNID(0xf27a9c51L, &sceIoRemove);
        linkFunctionByNID(0x779103a0L, &sceIoRename);
        linkFunctionByNID(0x1117c65fL, &sceIoRmdir);
        linkFunctionByNID(0x42ec03acL, &sceIoWrite);
        linkFunctionByNID(0xcd203292L, &sceKernelCancelEventFlag);
        linkFunctionByNID(0x8ffdf9a2L, &sceKernelCancelSema);
        linkFunctionByNID(0x812346e4L, &sceKernelClearEventFlag);
        linkFunctionByNID(0xe81caf8fL, &sceKernelCreateCallback);
        linkFunctionByNID(0x55c20a00L, &sceKernelCreateEventFlag);
        linkFunctionByNID(0xd6da4ba1L, &sceKernelCreateSema);
        linkFunctionByNID(0x446d8de6L, &sceKernelCreateThread);
        linkFunctionByNID(0x79d1c3faL, &sceKernelDcacheWritebackAll);
        linkFunctionByNID(0x34b9fa9eL, &sceKernelDcacheWritebackInvalidateRange);
        linkFunctionByNID(0x3ee30821L, &sceKernelDcacheWritebackRange);
        linkFunctionByNID(0xceadeb47L, &sceKernelDelayThread);
        linkFunctionByNID(0xedba5844L, &sceKernelDeleteCallback);
        linkFunctionByNID(0xef9e4c70L, &sceKernelDeleteEventFlag);
        linkFunctionByNID(0xed1410e0L, &sceKernelDeleteFpl);
        linkFunctionByNID(0x28b6489cL, &sceKernelDeleteSema);
        linkFunctionByNID(0x9fa03cd3L, &sceKernelDeleteThread);
        linkFunctionByNID(0x89b3d48cL, &sceKernelDeleteVpl);
        linkFunctionByNID(0x809ce29bL, &sceKernelExitDeleteThread);
        linkFunctionByNID(0xaa73c935L, &sceKernelExitThread);
        linkFunctionByNID(0xf6414a71L, &sceKernelFreeFpl);
        linkFunctionByNID(0xb736e9ffL, &sceKernelFreeVpl);
        linkFunctionByNID(0x293b45b8L, &sceKernelGetThreadId);
        linkFunctionByNID(0x30fd48f0L, &sceKernelPollEventFlag);
        linkFunctionByNID(0x58b1f937L, &sceKernelPollSema);
        linkFunctionByNID(0x090ccb3fL, &sceKernelPowerTick);
        linkFunctionByNID(0x1fb15a32L, &sceKernelSetEventFlag);
        linkFunctionByNID(0x3f53e640L, &sceKernelSignalSema);
        linkFunctionByNID(0xf475845dL, &sceKernelStartThread);
        linkFunctionByNID(0x383f7bccL, &sceKernelTerminateDeleteThread);
        linkFunctionByNID(0x402fcf22L, &sceKernelWaitEventFlag);
        linkFunctionByNID(0x4e3a1105L, &sceKernelWaitSema);
        linkFunctionByNID(0x278c0df5L, &sceKernelWaitThreadEnd);
        linkFunctionByNID(0x3f7ad767L, &sceRtcGetCurrentTick);
        linkFunctionByNID(0xc6395c03L, &vshCtrlReadBufferPositive);
        linkFunctionByNID(0x40716012L, &vshKernelExitVSHVSH);
        linkFunctionByNID(0xe13b6daeL, &sinf);
        linkFunctionByNID(0xeb6d20a5L, &cosf);
        linkFunctionByNID(0x09f5941eL, &sqrtf);
        linkFunctionByNID(0x60054052L, &rand);
        linkFunctionByNID(0x3fc9ae6aL, &sceKernelDevkitVersion);
        linkFunctionByNID(0x13f592bcL, &sceAudioOutputPannedBlocking);
        linkFunctionByNID(0xe2d56b2dL, &sceAudioOutputPanned);        
	   linkFunctionByNID(0x5ec81c55L, &sceAudioChReserve);
	   linkFunctionByNID(0x6fc46853L, &sceAudioChRelease);  
        linkFunctionByNID(0xab7592ffL, &memcpy); 
        linkFunctionByNID(0x81d0d1f7L, &memcmp);
        linkFunctionByNID(0xad8af84fL, &free);
        linkFunctionByNID(0x481c9adaL, &malloc);
        //linkFunctionByNID(0x6a2774f3L, &sceCtrlSetSamplingCycle);

        // un needed nids
        //linkFunctionByNID(0xcab439dfL, &printf);
        /*linkFunctionByNID(0x3624ae78L, &sceGuInit);
        linkFunctionByNID(0x5f01916fL, &sceGuStart);
        linkFunctionByNID(0xcb7dc84fL, &sceGuDrawBuffer);
        linkFunctionByNID(0xa79e81d7L, &sceGuDispBuffer);
        linkFunctionByNID(0xe4a45c4aL, &sceGuDepthBuffer);
        linkFunctionByNID(0x178a6887L, &sceGuOffset);
        linkFunctionByNID(0xe4e2ad90L, &sceGuViewport);
        linkFunctionByNID(0xf0e6ee76L, &sceGuDepthRange);
        linkFunctionByNID(0xb8730cb7L, &sceGuScissor);
        linkFunctionByNID(0x7e0eaddeL, &sceGuEnable);
        linkFunctionByNID(0xa24af273L, &sceGuDepthFunc);
        linkFunctionByNID(0x4b4f39fcL, &sceGuDisable);
        linkFunctionByNID(0x68522d68L, &sceGuFrontFace);
        linkFunctionByNID(0x5c5f727cL, &sceGuShadeModel);
        linkFunctionByNID(0x3a4d566bL, &sceGuTexMode);
        linkFunctionByNID(0x16ab1034L, &sceGuTexFunc);
        linkFunctionByNID(0x3ba5c5ccL, &sceGuTexFilter);
        linkFunctionByNID(0x7debf6faL, &sceGuTexScale);
        linkFunctionByNID(0xb07562abL, &sceGuTexOffset);
        linkFunctionByNID(0xba8cf6b9L, &sceGuFinish);
        linkFunctionByNID(0xe8d57dc6L, &sceGuSync);
        linkFunctionByNID(0x9b14983bL, &sceGuDisplay);
        linkFunctionByNID(0x9fd01858L, &sceGumMatrixMode);
        linkFunctionByNID(0xeb46c9c0L, &sceGumLoadIdentity);
        linkFunctionByNID(0x8f68eef4L, &sceGumPerspective);
        linkFunctionByNID(0xd9f56690L, &sceGuClearColor);
        linkFunctionByNID(0xa0547747L, &sceGuClearDepth);
        linkFunctionByNID(0x388d8736L, &sceGuClear);
        linkFunctionByNID(0x96610ae6L, &sceGumTranslate);
        linkFunctionByNID(0x82c8f916L, &sceGumRotateX);
        linkFunctionByNID(0xea4c24b3L, &sceGumDrawArray);
        linkFunctionByNID(0xL, &sceGuTexImage);
        linkFunctionByNID(0x1290e81aL, &sceGuSwapBuffers);
        /inkFunctionByNID(0xL, &sceGu);*/
        
        /***************************************************************************/
        /* Flush data cache.                                                       */
        /***************************************************************************/
        tiffsdkKernelDcacheWritebackAll();

        /***************************************************************************/
        /* We want to flush the ICache.  An easy way to do this is to flash the    */
        /* screen black and black.                                                 */
        /***************************************************************************/
        flashScreen(0, 0);
}
